/*
 *	LCD interface example
 *	This code will interface to a standard LCD controller
 *	like the Hitachi HD44780. It uses it in 4 bit mode, with
 *	the hardware connected as follows (the standard 14 pin 
 *	LCD connector is used):
 *	
 *	PORTB bits 0,1,3 and 4 are connected to the LCD data bits 4-7 (high nibble)
 *	PORTA bit 6 is connected to the LCD RS input (register select)
 *	PORTA bit 7 is connected to the LCD EN bit (enable)
 *	
 *	To use these routines, set up the port I/O (TRISA, TRISB) then
 *	call lcd_init(), then other routines as required.

History
air variable Nov.30,2008
	Correspond Hitech PIC C pro9.60pl4 build 3206 or higher
air variable Dec.6,2008
	add LCD location counter "lcd_loccnt"
	add LCD_cgwrite
	PIC liteDelay֐PICC Propɂ
	__delayp̃NbNݒlcd.hŐݒ
air variable Jan.12,2009
	fix lcd_init(void) routine

*/

#include 	<htc.h>
#include	"lcd.h"

static bit LCD_RS	@ ((unsigned)&PORTA*8+6);	// Register select (RA6)
static bit LCD_EN	@ ((unsigned)&PORTA*8+7);	// Enable (RA7)

#define	LCD_STROBE	(((LCD_EN = 1),(LCD_EN=0))	// spec. min 500ns

/* write a byte to the LCD in 4 bit mode */

void lcd_data_byteset(unsigned char c){
	PORTB = (PORTB & 0b11110100) |  (c & 0b00001011);		//Only low 4bit DATA
	if( (c & 0b00000100)== 0) RB4 = 0;
		else RB4=1;	//LCD D6 special process;
	LCD_STROBE;
}

void lcd_data_set(unsigned char c){
	PORTB = (PORTB & 0b11110100) | ( (c >>4 ) & 0b00001011 ) ;	//Hi SIDE DATA
	if( (c & 0b01000000)== 0) RB4 = 0;
		else RB4=1;	//LCD D6 special process
	LCD_STROBE;
	PORTB = (PORTB & 0b11110100) |  (c & 0b00001011 );		//low SIDE DATA
	if( (c & 0b00000100)== 0) RB4 = 0;
		else RB4=1;	//LCD D6 special process;
	LCD_STROBE;
}

void
lcd_write(unsigned char c)
{
	LCD_RS = 0;
	lcd_data_set(c);
	__delay_us(40);
}

/*
 * 	Clear and home the LCD
 */

void
lcd_clear(void)
{
	lcd_write(0x1);
	lcd_loccnt=0;
	__delay_ms(2);
}

/* write a string of chars to the LCD */

void
lcd_puts(const unsigned char * s)
{
	while(*s)
		lcd_putch(*s++);
}

/* write one character to the LCD and auto increment location counter*/

void
lcd_putch(unsigned char c)
{
	LCD_RS = 1;	// write characters
	lcd_data_set(c);
	__delay_us(40);

	lcd_loccnt++;
	if (lcd_loccnt == 0x10){//1iڍs
		lcd_loccnt = 0x40;
		lcd_goto(lcd_loccnt);
	}
	if (lcd_loccnt == 0x50){//2iڍs
		lcd_loccnt = 0x00;
		lcd_goto(lcd_loccnt);
	}
}

//CG RAM[ݗp
void
lcd_cgwrite(unsigned char c)
{
	LCD_RS = 1;	// write characters
	lcd_data_set(c);
	__delay_us(40);
}

/*
 * Go to the specified position
 */
void
lcd_goto(unsigned char pos)
{
	lcd_write(0x80+pos);
	lcd_loccnt=pos;
}
	
/* initialise the LCD - put into 4 bit mode */
void
lcd_init(void)
{
	LCD_RS = 0;				// write control bytes
	__delay_ms(15);			// power on delay (PICC Pro)
	lcd_data_byteset(0x03);	// attention!
	__delay_ms(5);			//wait
	LCD_STROBE;				//strobe again
	__delay_us(100);		//wait
	LCD_STROBE;				//strobe again
	__delay_ms(5);			//wait
	lcd_data_byteset(0x02) ;// set 4 bit mode
	__delay_us(100);		//wait
	lcd_write(0x28);		// 4 bit mode, 1/16 duty, 5x8 font
	lcd_write(0x08);		// display off
	lcd_write(0x0F);		// display on, blink curson on
	lcd_write(0x06);		// entry mode
	lcd_loccnt=0;
}
